#!/usr/bin/env python

import glob, os, sys

# The next statements assure importation of qwt and qwtc from the
# local source tree. This is useful for debugging and testing.
globs = glob.glob('../build/lib.*/qwt')
if globs:                               # found PyQwt modules made by distutils
    sys.path.insert(0, os.path.split(globs[0])[0])

from qt import *
from qwt import *
from Numeric import *
from LinearAlgebra import *

# realistic
# all cv's have width = 5.5 mm for 0.55 pF/mm**2

filename = None
case = 1
if case == 1:
    # screen leaving a space of 1.5 mm at 4.2 K
    cv = 15.125e-12 # for height = 5.0 mm
    lv =  6.50e-9
    lh =  6.40e-9
    ch, lq = 1.00e-12, 1.0/1000
    #filename = 'LCR-DATA/HEEC15C5.TXT'
    filename = 'LCR-DATA/HE15C5OK.TXT'
elif case == 2:
    # without screen at 4.2 K
    cv = 10.5875e-12 # for heigth = 3.5 mm
    lv = 6.00e-9
    lh = 13.5e-9 # a big and surprising difference!!
    ch, lq = 1.00e-12, 1.0/1000
    filename = 'LCR-DATA/HECAPA35.TXT'
elif case == 3:
    # without screen at 4.2 K
    cv = 15.88125e-12 # for height = 5.25 mm
    lv = 6.00e-9
    lh = 14.5e-9
    ch, lq = 2.0e-12, 1.0/1000
    filename = 'LCR-DATA/HC525.TXT'
elif case == 4:
    # with screen at 4.2 K
    cv = 15.88125e-12 # for height = 5.25 mm
    lv = 6.56e-9
    lh = 6.44e-9
    ch, lq = 4.0e-12, 1.0/300
    filename = 'LCR-DATA/HC525E2.TXT'

def read_reflection_data(filename):
    file = open(filename, 'r')
    lines = file.readlines()[14:]
    file.close
    f = zeros(len(lines), Float)
    x = zeros(len(lines), Float)
    y = zeros(len(lines), Float)
    i = 0
    for line in lines:
        words = line.split()
        f[i] = float(words[0])
        x[i] = float(words[1])
        y[i] = float(words[2])
        i += 1
    return f, x, y

def g_c(value, omega):
    return 1j*omega*value

def g_l(value, omega):
    return 1.0/((lq+1j)*omega*value)

def g_r(value, omega):
    return 1.0/value

def rhs(omega):
    """based on Eq. 1 of Hayes et al. Journal of Magnetic Resonance 63?"""
    return 2*sqrt((1-omega*omega*cv*lv)/(lh*cv))

# input node = -1
# ground node = -2

def zMatrix(circuit, omega):
    # find the number of modes and the node connected to the "input" branch
    highs = []
    for func, lo, hi, value in circuit:
        highs.append(hi)
        if lo == -1:
            inNode = hi
    nodes = 1 + max(highs)
    # the hard work
    z = zeros(len(omega), Complex)
    for i in range(len(omega)):
        a = zeros((nodes, nodes), Complex)  # nodal conductance matrix
        b = zeros(nodes, Complex)           # source current vector
        for func, lo, hi, value in circuit:
            g = func(value, omega[i])
            if lo == -2:                    # ground
                a[hi][hi] += g
            elif lo == -1:                  # input
                b[hi] += g
                a[hi][hi] += g
            else:
                a[lo][lo] += g
                a[hi][hi] += g
                a[lo][hi] -= g
                a[hi][lo] -= g
        v = solve_linear_equations(a, b)
        z[i] = 1.0/(b[inNode]*(1-v[inNode]))
    return z


class SmithChart(QWidget):
    def __init__(self,*args):
        apply(QWidget.__init__,(self,) + args)
	
	# The QwtPlot widget
	self.griPlot = QwtPlot("Smith Chart", self)
        self.gnfPlot = QwtPlot("abs(Gamma) - f", self)
        self.zPlot = QwtPlot("re(Z), im(Z) - f", self)
        self.rPlot = QwtPlot("Intersect Plot", self)
        # FIXME : look in .sip for the last option (step): maybe double = 0.0
        self.griPlot.setAxisScale(QwtPlot.xBottom, -1.0, 1.0)
        self.griPlot.setAxisScale(QwtPlot.yLeft, -1.0, 1.0)
	#self.zPlot.setAxisScale(QwtPlot.xBottom, 300, 400)
        
        # circuit description
        circuit = [
            (g_c, -1,  0, ch),
            # rung 1
            (g_c,  0,  1, cv),
            (g_l, -2,  0, lv),
            (g_l,  1,  3, lh),
            # rung 2
            (g_c,  2,  3, cv),
            (g_l, -2,  2, lv),
            (g_l,  3,  5, 0.8*lh),
            # rung 3
            (g_c,  4,  5, cv),
            (g_l, -2,  4, lv),
            (g_l,  5,  7, lh),
            # rung 4
            (g_c,  6,  7, cv),
            (g_l, -2,  6, lv),
            (g_l,  7,  9, lh),
            # rung 5
            (g_c,  8,  9, cv),
            (g_l, -2,  8, lv),
            (g_l,  9, 11, lh),
            # rung 6
            (g_c, 10, 11, cv),
            (g_l, -2, 10, lv),
            (g_l,  1, 11, lh),
            ]

        fmax = 1.0/(2*pi*sqrt(lv*cv))
        #fs = 2*pi*arrayrange(fmax/2, fmax, 100e3)
        fs = 2*pi*arrayrange(fmax/2, fmax, fmax/1600)
        zs = zMatrix(circuit, fs)
        gs = (zs-50.0)/(zs+50.0)
        sect = rhs(fs)/(2e6*pi)
        fs = fs/(2e6*pi)

        fa2 = 1.0/(4*pi*pi*(lh+2*lv)*cv)
        fb2 = 1.0/(4*pi*pi*lv*cv)
        for i in [1, 2, 3]:
            s = sin(pi*i/6)
            c = cos(2*pi*i/6)
            #print s/(pi*(sqrt(cv*(lh+2*lv*s)))) # wrong
            print s/(pi*(sqrt(cv*(lh+4*lv*s*s)))) # ok
            #print sqrt(2*fa2*(1-c)/(1-2*fa2*c/fb2)) # ok, Tropp

        curveCalSnail = self.griPlot.insertCurve("im(Gamma) vs re(Gamma)")
        self.griPlot.setCurveData(curveCalSnail, gs.real, gs.imag)
        self.griPlot.setCurvePen(curveCalSnail, QPen(Qt.black))
        self.griPlot.replot()
        
        curveCalAbsG = self.gnfPlot.insertCurve("cal. abs(Gamma) vs f")
        self.gnfPlot.setCurveData(curveCalAbsG, fs, abs(gs))
        self.gnfPlot.setCurvePen(curveCalAbsG, QPen(Qt.black))
        self.gnfPlot.setAxisScale(QwtPlot.xBottom, fs[0], fs[-1])
        self.gnfPlot.replot()
        
        curveReLadder = self.zPlot.insertCurve("re(zLadder)")
        curveImLadder = self.zPlot.insertCurve("im(zLadder)")
        self.zPlot.setCurveData(curveReLadder, fs, zs.real)
        self.zPlot.setCurvePen(curveReLadder, QPen(Qt.red))
        self.zPlot.setCurveData(curveImLadder, fs, zs.imag)
        self.zPlot.setCurvePen(curveImLadder, QPen(Qt.blue))
        self.zPlot.replot()

        curveSect1 = self.rPlot.insertCurve("mode 1")
        curveSect2 = self.rPlot.insertCurve("mode 2")
        curveSect3 = self.rPlot.insertCurve("mode 3")
        curveSectR = self.rPlot.insertCurve("intersect")
        self.rPlot.setCurveData(curveSectR, fs, fs)
        self.rPlot.setCurvePen(curveSectR, QPen(Qt.red))
        self.rPlot.setCurveData(curveSect1, fs, sin(pi/6)*sect)
        self.rPlot.setCurvePen(curveSect1, QPen(Qt.green))
        self.rPlot.setCurveData(curveSect2, fs, sin(pi/3)*sect)
        self.rPlot.setCurvePen(curveSect2, QPen(Qt.green))
        self.rPlot.setCurveData(curveSect3, fs, sin(pi/2)*sect)
        self.rPlot.setCurvePen(curveSect3, QPen(Qt.green))
        self.rPlot.replot()

        print filename
        if filename:
            fs, gxs, gys = read_reflection_data(filename)
            fs /= 1e6
            gs = gxs+1j*gys
            
            curveExpSnail = self.griPlot.insertCurve("im(Gamma) vs re(Gamma)")
            self.griPlot.setCurveData(curveExpSnail, gs.real, gs.imag)
            self.griPlot.setCurvePen(curveExpSnail, QPen(Qt.red))
            self.griPlot.replot()
        
            curveExpAbsG = self.gnfPlot.insertCurve("exp. abs(Gamma) vs f")
            self.gnfPlot.setCurveData(curveExpAbsG, fs, abs(gs))
            self.gnfPlot.setCurvePen(curveExpAbsG, QPen(Qt.red))
            #self.gnfPlot.setAxisScale(QwtPlot.xBottom, 350, 50)
            self.gnfPlot.replot()


    def resizeEvent(self, e):
        x = e.size().width()/2
        y = e.size().height()/2
    	self.griPlot.resize(x, y)
	self.griPlot.move(0, 0)
    	self.gnfPlot.resize(x, y)
	self.gnfPlot.move(0, y)
        self.zPlot.resize(x, y)
        self.zPlot.move(x, 0)
        self.rPlot.resize(x, y)
        self.rPlot.move(x, y)
	

if __name__ == '__main__':
    app = QApplication(sys.argv)
    chart = SmithChart()
    app.setMainWidget(chart)
    chart.resize(700, 700)
    chart.show()
    app.exec_loop()
