#!/usr/bin/env python

import glob, os, sys

# The next statements assure importation of qwt from the
# local source tree. This is useful for debugging and testing.
globs = glob.glob('../build/lib.*/qwt')
if globs:                               # found PyQwt modules made by distutils
    sys.path.insert(0, os.path.split(globs[0])[0])

from qt import *
from qwt import *
from Numeric import *

inductor = 84e-9
resistor = 0.7
capaP = 2.235e-12
capaS = 0.33e-12

def argument(z):
    return arctan(z.imaginary/z.real)

def norm(x, y):
    return sqrt(x*x+y*y)

# element[0] in
# 'L' for inductor
# 'R' for resistor
# 'C' for capacitor
# 'S' for short
# 'O' for open
# 'A' for assembled circuit
# element[1] in
# 's' for 2-port-series,
# 'p' for 2-port-parallel,
# 't' for transmission line (4 port)
def zLadder(circuit, omega):
    head = circuit[0]
    if head[1] == 's':
        v, i = 0.0, 1.0
    else:
        v, i = 1.0, 0.0
    for element in circuit:
        if element[1] == 's':
            if element[0] == 'L': z = 1j*omega*element[2]
            elif element[0] == 'R': z = element[2]
            elif element[0] == 'C': z = -1j/(omega*element[2])
            elif element[0] == 'S': z = 0
            elif element[0] == 'A': z = zLadder(element[2], omega)
            v = i*z+v
        elif element[1] == 'p':
            if element[0] == 'L': y = -1j/(omega*element[2])
            elif element[0] == 'R': y = 1.0/element[2]
            elif element[0] == 'C': y = 1j*omega*element[2]
            elif element[0] == 'O': y = 0
            elif element[0] == 'A': y = 1.0/zLadder(element[2], omega)
            i = v*y+i
        elif element[1] == 't': # FIXME: freq?
            z = element[2]
            x = (element[3]*sqrt(freq/130e6)+1j*omega/element[4])*element[5]
            a, b = cosh(x), sinh(x)
            v, i = a*v+z*b*i, b*v/z+a*i
        else:
            print "ERROR"
    return v/i

def zPython(l, r, cp, cs, omega):
    return 1/(1j*omega*cp+1/(r+1j*omega*l))+1/(1j*omega*cs)

class SmithChart(QWidget):
    def __init__(self,*args):
        apply(QWidget.__init__,(self,) + args)
	
	# The QwtPlot widget
	self.gPlot = QwtPlot("Smith Chart", self)
        self.zPlot = QwtPlot("Impedance", self)
        # FIXME : look in .sip for the last option (step): maybe double = 0.0
        self.gPlot.setAxisScale(QwtPlot.xBottom, -1.0, 1.0)
        self.gPlot.setAxisScale(QwtPlot.yLeft, -1.0, 1.0)
	self.zPlot.setAxisScale(QwtPlot.xBottom, 300, 400)

        # circuit description
        Circuit4 = [
            ['L', 's', inductor],
            ['R', 's', resistor],
            ['C', 'p',  capaP],
            ['C', 's', capaS],
            ]

        # calculate impedance and reflection coefficient
        fs = 2*pi*arrayrange(130e6, 770e6, 0.25e6)
        zs = zLadder(Circuit4, fs)
        ys = zPython(inductor, resistor, capaP, capaS, fs)
        gs = (zs-50.0)/(zs+50.0)
        fs = fs/(2e6*pi)

        gamma = self.gPlot.insertCurve("im(Gamma) vs re(Gamma)")
        self.gPlot.setCurveData(gamma, gs.real, gs.imag)
        curveReLadder = self.zPlot.insertCurve("re(zLadder)")
        curveImLadder = self.zPlot.insertCurve("im(zLadder)")
        curveRePython = self.zPlot.insertCurve("re(zPython)")
        curveImPython = self.zPlot.insertCurve("im(zPython)")
        self.zPlot.setCurveData(curveReLadder, fs, zs.real)
        self.zPlot.setCurvePen(curveReLadder, QPen(Qt.red))
        self.zPlot.setCurveData(curveImLadder, fs, zs.imag)
        self.zPlot.setCurvePen(curveImLadder, QPen(Qt.red))
        self.zPlot.setCurveData(curveRePython, fs, ys.real)
        self.zPlot.setCurvePen(curveRePython, QPen(Qt.blue))
        self.zPlot.setCurveData(curveImPython, fs, ys.imag)
        self.zPlot.setCurvePen(curveImPython, QPen(Qt.blue))

    def resizeEvent(self, e):
        x = e.size().width()/2
        y = e.size().height()
    	self.gPlot.resize(x, y)
	self.gPlot.move(0, 0)
        self.zPlot.resize(x, y)
        self.zPlot.move(x, 0)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    chart = SmithChart()
    app.setMainWidget(chart)
    chart.resize(800, 400)
    chart.show()
    app.exec_loop()






